#!/usr/bin/env python

# Placeholder script for a test suite that confirms the CheesePi distribution sitting
# in the current version of the repository is valid and has no obvious problems.
# It will return and error code (>0) if there is a problem, and 0 if things appear ok.

import sys


# seems ok
sys.exit(0)
